; struct.inc

  list c=200, n=0, b=8 ; c columns, n lines, b tabsize

; General definitions for 12- and 14-bit core Microchip PIC microcontrollers
; (c) PROPIX  Written by Pinter Gabor
; H-8000 Szekesfehervar, Krivanyi u. 15.
; Tel: +36 30 9972445, +36 22 300915
; Fax: +36 22 304326
; Email: propix@freemail.hu, Pinter.Gabor@freemail.hu
; Web: http://www.propix.hu

; Revisions:
;  V1.0  11/21/2000  New with lots of comments

; Tabsize=8

; Assembler settings
		radix dec		; Default radix is of course decimal
		list f=inhx8m		; Intel hex
		errorlevel 0, -207, -205
		;nolist
		;noexpand

;-----------------------------------------------
; Move literal to any register
; Syntax:
;	movlf immediate,register
; Registers: w
;
;-----------------------------------------------
; Move literal to w and then w to the destination register.
;-----------------------------------------------
movlf		macro immediate,register
				movlw immediate
				movwf register
 		endm


;-----------------------------------------------
; Move any register to any register
; Syntax:
;	movff source,dest
; Registers: w
;
;-----------------------------------------------
; Move the source register to w and then w to the destination register.
;-----------------------------------------------

movff		macro source,dest
				movfw source
				movwf dest
		endm


;-----------------------------------------------
; Return
; Syntax:
;	ret
; Registers: -
;
;-----------------------------------------------
; Shorthand. ret maps to retlw on 12-bit core and to return on 14-bit core processors.
;-----------------------------------------------

ret		macro
				return
		endm


;-----------------------------------------------
; Skips
; Syntax:
;	skpc
;	skpnc
;	skpz
;	skpnz
;	skpb register,bit
;	skpnb register,bit
; Registers: -
;
;-----------------------------------------------
; skpc, skpnc, skpz, skpnz are assembler pseudoinstructions.
; skpb and skpnb are defined here.
;-----------------------------------------------

skpb		macro register,bit
				btfss register,bit
		endm

skpnb		macro register,bit
				btfsc register,bit
		endm


;-----------------------------------------------
; Double nop
; Syntax:
;	nop2
; Registers: -
;
;-----------------------------------------------
; A nop that executes in two instruction cycles, but uses only one word.
; The page select register must point to the current page to operate properly.
; They work the same way: The next instruction is skipped if the tested bit is true.
;-----------------------------------------------

nop2		macro
				goto $+1
 		endm


;===============================================
; Jump instructions
;
; Syntax1:
;	b<|z|nz|c|nc> address
;
; Syntax2:
;	b<b|nb> register,bit,address
;
; Registers: -
;
;-----------------------------------------------
; b, bc, bnc, bz, bnz are assembler pseudoinstructions.
; bb and bnb are defined here.
; They work the same way: The jump is taken if the tested bit is true.
;-----------------------------------------------

bb		macro register,bit,address
				btfsc register,bit
				  goto address
		endm

bnb		macro register,bit,address
				btfss register,bit
				  goto address
		endm


;===============================================
; Decrement/Increment register and jump if zero
;
; Syntax:
;	dbnz register,address
;	ibnz register,address
;
; Registers: -
;
;-----------------------------------------------
; Decrement/increment register and jump if the result is zero.
; Flags are not changed.
;-----------------------------------------------

dbnz		macro register,address
				decfsz register,f
				goto address
		endm

ibnz		macro register,address
				incfsz register,f
				goto address
		endm


;===============================================
; Conditional rets
;
; Syntax:
;	ret_<z,nz,c,nc> 			; Return if condition is true
;	ret_<b,nb> register, flag		; Return if bit is set/reset
;	retlw_<z,nz,c,nc> 			; Return value if condition is true
;	retlw_<b,nb> register, flag, value	; Return value if bit is set/reset
;
; Registers: (w)
;
;-----------------------------------------------
; Return if the tested bit is true.
; ret_ returns 0 on 12-bit core, nothing on 14-bit core processors
; retlw_ returns the value parameter
;-----------------------------------------------

ret_z		macro
				skpnz
				  return
		endm

ret_nz		macro
				skpz
				  return
		endm

ret_c		macro
				skpnc
				  return
		endm

ret_nc		macro
				skpc
				  return
		endm

ret_b		macro register,flag
				btfsc register,flag
				  return
		endm

ret_nb		macro register,flag
				btfss register,flag
				  return
		endm

retlw_z		macro value
				skpnz
				  retlw value
		endm

retlw_nz	macro value
				skpz
				  retlw value
		endm

retlw_c		macro value
				skpnc
				  retlw value
		endm

retlw_nc 	macro value
				skpc
				  retlw value
		endm

retlw_b 	macro register,flag,value
				btfsc register,flag
				  retlw value
		endm

retlw_nb 	macro register,flag,value
				btfss register,flag
				  retlw value
		endm


;===============================================
; Loop structures

; Syntax1:
;	loop
;	  ...
;	endloop_<z|nz|c|nc>
;
; Syntax2:
;	loop
;	  ...
;	endloop_<b|nb> register,flag
;
; Syntax3:
;	loop
;	  ...
;	endloop[_inc] register
;
; Registers: -
;
;-----------------------------------------------
; Instructions between the loop and endloop macros are executed repeatedly.
; The loop is terminated if the	condition is true.
; The condition is either a test bit or a register decrement.
; Loops can be nested.
;-----------------------------------------------


		__level=0
		__loop=0

loop		macro
__loop#v(__loop)
				__endloop#v(__level)=__loop
				__loop++
				__level++
		endm

endloop_infinite macro
				__level--
				goto __loop#v(__endloop#v(__level))
		endm

endloop_z	macro
				__level--
				skpz
				  goto __loop#v(__endloop#v(__level))
		endm

endloop_nz	macro
				__level--
				skpnz
				  goto __loop#v(__endloop#v(__level))
		endm

endloop_c	macro
				__level--
				skpc
				  goto __loop#v(__endloop#v(__level))
		endm

endloop_nc	macro
				__level--
				skpnc
				  goto __loop#v(__endloop#v(__level))
		endm

endloop_b	macro register,bit
				__level--
				btfss	register,bit
				  goto __loop#v(__endloop#v(__level))
		endm

endloop_nb	macro register,bit
				__level--
				btfsc	register,bit
				  goto __loop#v(__endloop#v(__level))
		endm

endloop		macro register
				__level--
				decfsz register,f
				  goto __loop#v(__endloop#v(__level))
		endm

endloop_inc	macro register
				__level--
				incfsz register,f
				  goto __loop#v(__endloop#v(__level))
		endm



;===============================================
; Conditional structures
;
; Syntax1:
;	if_<z|nz|c|nc>
;	  ...
;	mendif
;
; Syntax2:
;	if_<b,nb> register,bit
;	  ...
;	mendif
;
; Syntax3:
;	if_<z|nz|c|nc>
;	  ...
;	melse
;	  ...
;	mendif
;
; Syntax4:
;	if_<b,nb> register,bit
;	  ...
;	melse
;	  ...
;	mendif
;
; Syntax5:
;	if_<b,nb>_<b,nb> register1,bit1, register2,bit2
;	  ...
;	melse
;	  ...
;	mendif
;
; Registers: -
;
;-----------------------------------------------
; Instructions between if and melse (or mendif) are executed if the condition is true.
; Instructions between melse and (or mendif) are executed if the condition is true.
; Conditionals can be nested.
; Syntax 5 can test two bits, and faster than two other ifs.
;-----------------------------------------------

		__if=0

if_z		macro
				skpz
				  goto __if#v(__if)
				__endif#v(__level)=__if
				__if++
				__level++
		endm

if_nz		macro
				skpnz
				  goto __if#v(__if)
				__endif#v(__level)=__if
				__if++
				__level++
		endm

if_c		macro
				skpc
				  goto __if#v(__if)
				__endif#v(__level)=__if
				__if++
				__level++
		endm

if_nc		macro
				skpnc
				  goto __if#v(__if)
				__endif#v(__level)=__if
				__if++
				__level++
		endm

if_b		macro register,bit
				btfss register,bit
				  goto __if#v(__if)
				__endif#v(__level)=__if
				__if++
				__level++
		endm

if_nb		macro register,bit
				btfsc register,bit
				  goto __if#v(__if)
				__endif#v(__level)=__if
				__if++
				__level++
		endm

if_b_b		macro register1,bit1,register2,bit2
				  btfsc register1,bit1
				    btfss register2,bit2
				      goto __if#v(__if)
				  __endif#v(__level)=__if
				  __if++
				  __level++
		endm


if_b_nb		macro register1,bit1,register2,bit2
				  btfsc register1,bit1
				    btfsc register2,bit2
				      goto __if#v(__if)
				  __endif#v(__level)=__if
				  __if++
				  __level++
		endm


if_nb_b		macro register1,bit1,register2,bit2
				  btfss register1,bit1
				    btfss register2,bit2
				      goto __if#v(__if)
				  __endif#v(__level)=__if
				  __if++
				  __level++
		endm


if_nb_nb	macro register1,bit1,register2,bit2
				  btfss register1,bit1
				    btfsc register2,bit2
				      goto __if#v(__if)
				  __endif#v(__level)=__if
				  __if++
				  __level++
		endm


melse		macro
				goto __if#v(__if)
__if#v(__endif#v(__level-1))
				__endif#v(__level-1)=__if
				__if++
		endm

mendif		macro
				__level--
__if#v(__endif#v(__level))
		endm


;===============================================
; Local label generator
; Syntax: label#v(LL)
;	  LL++
;
;-----------------------------------------------
; If none of the conditional structures can be used, generate a local label
;   label#v(LL)
; where label is any valid label name.
; At the end of the block or procedure place
;   L++
; The generated code block can safely copied and edited.
;-----------------------------------------------

	LL=1000


		list



